﻿using System;
using System.Diagnostics;

namespace ConsoleApp2
{
    class MyType
    {
        // public Span<byte> ByteBuffer;
        public Memory<byte> MemoryBuffer;
    }

    class Program
    {
        static void Main(string[] args)
        {
            /*
            {
                byte[] buffer = new byte[1000];

                MyType type = new MyType();
                type.MemoryBuffer = buffer;

                Span<byte> fastBuf = type.MemoryBuffer.Span;

                for (int i = 0; i < fastBuf.Length; i ++)
                {
                    // ... 
                }
            }
            */

            Action<int, string, Func<byte[], int>, byte[]> action = (loopCount, title, work, arg) =>
            {
                Stopwatch st = new Stopwatch();
                st.Start();

                Random rand = new Random(Environment.TickCount);

                for (int i = 0; i < loopCount; i++)
                {
                    work(arg);
                }

                st.Stop();

                Console.WriteLine(title + " : " + st.ElapsedMilliseconds);
            };

            byte[] buf = new byte[1];

            action(1, "touch-JIT", ForLoop, buf);
            action(1, "touch-JIT", MemoryLoop, buf);
            action(1, "touch-JIT", MemorySpanLoop, buf);
            action(1, "touch-JIT", PtrLoop, buf);

            Console.WriteLine();

            buf = new byte[10000];
            action(100000, "ForLoop", ForLoop, buf);
            action(100000, "MemoryLoop", MemoryLoop, buf);
            action(100000, "MemorySpanLoop", MemorySpanLoop, buf);
            action(100000, "PtrLoop", PtrLoop, buf);
        }

        static int ForLoop(byte[] buffer)
        {
            int sum = 0;

            for (int i = 0; i < buffer.Length; i++)
            {
                sum += buffer[i];
            }

            return sum;
        }

        static int MemoryLoop(byte[] buffer)
        {
            Memory<byte> memory = buffer;

            int sum = 0;
            for (int i = 0; i < memory.Span.Length; i++)
            {
                sum += memory.Span[i];
            }

            return sum;
        }

        static int MemorySpanLoop(byte[] buffer)
        {
            Memory<byte> memory = buffer;

            Span<byte> span = memory.Span;
            int sum = 0;
            for (int i = 0; i < span.Length; i++)
            {
                sum += span[i];
            }

            return sum;
        }

        static unsafe int PtrLoop(byte[] buffer)
        {
            int sum = 0;
            fixed (byte* ptr = buffer)
            {
                for (int i = 0; i < buffer.Length; i++)
                {
                    sum += *(ptr + i);
                }
            }
            return sum;
        }
    }
}
